import com.imsl.math.*;

// Documentation example

public class ComplexSuperLUEx1
{
	public static void main(String args[]) throws Exception
	{
		int m;
		ComplexSuperLU ComplexSparseLU;
		double conditionNumber, recip_pivot_growth;
		Complex[] sol = null;
		double Ferr, Berr;

		Complex[] b1 = { 
                    new Complex(3.0, 17.0), new Complex(-19.0, 5.0), 
                    new Complex(6.0, 18.0), new Complex(-38.0, 32.0), 
                    new Complex(-63.0, 49.0), new Complex(-57.0, 83.0)};

		Complex[] b2 = { 
                    new Complex(54.0, -112.0), new Complex(46.0, -58.0), 
                    new Complex(12.0, 0.0), new Complex(5.0, -51.0), 
                    new Complex(78.0, 34.0), new Complex(60.0, -94.0)};


		// Initialize input matrix A.
		m = 6;
		ComplexSparseMatrix a = new ComplexSparseMatrix(m,m);

		a.set(0, 0, new Complex(10.0, 7.0));
		a.set(1, 1, new Complex(3.0, 2.0));
		a.set(1, 2, new Complex(-3.0, 0.0));
		a.set(1, 3, new Complex(-1.0, 2.0));
		a.set(2, 2, new Complex(4.0, 2.0));
		a.set(3, 0, new Complex(-2.0, -4.0));
		a.set(3, 3, new Complex(1.0, 6.0));
		a.set(3, 4, new Complex(-1.0, 3.0));
		a.set(4, 0, new Complex(-5.0, 4.0));
		a.set(4, 3, new Complex(-5.0, 0.0));
		a.set(4, 4, new Complex(12.0, 2.0));
		a.set(4, 5, new Complex(-7.0, 7.0));
		a.set(5, 0, new Complex(-1.0, 12.0));
		a.set(5, 1, new Complex(-2.0, 8.0));
		a.set(5, 5, new Complex(3.0, 7.0));

		// Compute the sparse LU factorization of a

		ComplexSparseLU = new ComplexSuperLU(a);

		ComplexSparseLU.setEquilibrate(false);
		ComplexSparseLU.setColumnPermutationMethod(
                        ComplexSuperLU.NATURAL_ORDERING);
		ComplexSparseLU.setPivotGrowth(true);

		// Set option of iterative refinement
		ComplexSparseLU.setIterativeRefinement(true);

		// Solve sparse system A*x = b1
		System.out.println();
		System.out.println("Solve sparse System Ax=b1");
		System.out.println("=========================");
		System.out.println();

		sol = ComplexSparseLU.solve(b1);
		new PrintMatrix("Solution").print(sol);

		Ferr = ComplexSparseLU.getForwardErrorBound();
		Berr = ComplexSparseLU.getRelativeBackwardError();

		System.out.println();
		System.out.println("Forward error bound: "+Ferr);
		System.out.println("Relative backward error: "+ Berr);
		System.out.println();
		System.out.println();

		// Solve sparse system A^H*x = b2
		System.out.println();
		System.out.println("Solve sparse System A^Hx=b2");
		System.out.println("===========================");
		System.out.println();

		sol = ComplexSparseLU.solveConjugateTranspose(b2);
		new PrintMatrix("Solution").print(sol);

		Ferr = ComplexSparseLU.getForwardErrorBound();
		Berr = ComplexSparseLU.getRelativeBackwardError();

		System.out.println();
		System.out.println("Forward error bound: "+Ferr);
		System.out.println("Relative backward error: "+ Berr);
		System.out.println();
		System.out.println();

		// Compute reciprocal pivot growth factor and condition number

		recip_pivot_growth = 
                        ComplexSparseLU.getReciprocalPivotGrowthFactor();
		conditionNumber = ComplexSparseLU.getConditionNumber();

		System.out.println("Pivot growth factor and condition number");
		System.out.println("========================================");
		System.out.println();
		
		System.out.println("Reciprocal pivot growth factor: "+ 
                        recip_pivot_growth);
		System.out.println("Reciprocal condition number: "+ 
                        conditionNumber);
		System.out.println();
	}	
}